'use strict'
Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' })
const naturalCompare = require('natural-compare-lite')
let compare = (a, b, options) => {
  var _a, _b
  if ((_a = b.dependencies) == null ? void 0 : _a.includes(a.name)) {
    return -1
  } else if ((_b = a.dependencies) == null ? void 0 : _b.includes(b.name)) {
    return 1
  }
  let orderCoefficient = options.order === 'asc' ? 1 : -1
  let sortingFunction
  let formatString =
    options.type === 'line-length' || !options.ignoreCase
      ? string => string
      : string => string.toLowerCase()
  let nodeValueGetter = options.nodeValueGetter ?? (node => node.name)
  if (options.type === 'alphabetical') {
    sortingFunction = (aNode, bNode) =>
      formatString(nodeValueGetter(aNode)).localeCompare(
        formatString(nodeValueGetter(bNode)),
      )
  } else if (options.type === 'natural') {
    let prepareNumeric = string => {
      let formattedNumberPattern = /^[+-]?[\d ,_]+(\.[\d ,_]+)?$/
      if (formattedNumberPattern.test(string)) {
        return string.replaceAll(/[ ,_]/g, '')
      }
      return string
    }
    sortingFunction = (aNode, bNode) =>
      naturalCompare(
        prepareNumeric(formatString(nodeValueGetter(aNode))),
        prepareNumeric(formatString(nodeValueGetter(bNode))),
      )
  } else {
    sortingFunction = (aNode, bNode) => {
      let aSize = aNode.size
      let bSize = bNode.size
      let { maxLineLength } = options
      if (maxLineLength) {
        let isTooLong = (size, node) =>
          size > maxLineLength && node.hasMultipleImportDeclarations
        if (isTooLong(aSize, aNode)) {
          aSize = nodeValueGetter(aNode).length + 10
        }
        if (isTooLong(bSize, bNode)) {
          bSize = nodeValueGetter(bNode).length + 10
        }
      }
      return aSize - bSize
    }
  }
  return orderCoefficient * sortingFunction(a, b)
}
exports.compare = compare
